@@ -232,7 +232,7 @@ class Node
      * node, it will return the table and database nodes. The names of the returned
      * nodes can be used in SQL queries, etc...
      *
-     * @return Node
+     * @return Node|false
      */
     public function realParent()
     {
@@ -271,8 +271,9 @@ class Node
     }
 
     /**
-     * Returns true the node has some siblings (other nodes on the same tree level,
-     * in the same branch), false otherwise. The only exception is for nodes on
+     * Returns true if the node has some siblings (other nodes on the same tree
+     * level, in the same branch), false otherwise.
+     * The only exception is for nodes on
      * the third level of the tree (columns and indexes), for which the function
      * always returns true. This is because we want to render the containers
      * for these nodes
@@ -365,161 +366,183 @@ class Node
     public function getData($type, $pos, $searchClause = '')
     {
         $maxItems = $GLOBALS['cfg']['FirstLevelNavigationItems'];
-        if ($GLOBALS['cfg']['NavigationTreeEnableGrouping']) {
-            $dbSeparator = $GLOBALS['cfg']['NavigationTreeDbSeparator'];
-            if (! $GLOBALS['cfg']['Server']['DisableIS']) {
-                $query  = "SELECT `SCHEMA_NAME` ";
-                $query .= "FROM `INFORMATION_SCHEMA`.`SCHEMATA`, ";
-                $query .= "(";
-                $query .= "SELECT DB_first_level ";
-                $query .= "FROM ( ";
-                $query .= "SELECT DISTINCT SUBSTRING_INDEX(SCHEMA_NAME, ";
-                $query .= "'$dbSeparator', 1) ";
-                $query .= "DB_first_level ";
-                $query .= "FROM INFORMATION_SCHEMA.SCHEMATA ";
-                $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
-                $query .= ") t ";
-                $query .= "ORDER BY DB_first_level ASC ";
-                $query .= "LIMIT $pos, $maxItems";
-                $query .= ") t2 ";
-                $query .= "WHERE 1 = LOCATE(CONCAT(DB_first_level, ";
-                $query .= "'$dbSeparator'), ";
-                $query .= "CONCAT(SCHEMA_NAME, ";
-                $query .= "'$dbSeparator')) ";
-                $query .= "ORDER BY SCHEMA_NAME ASC";
-                $retval = $GLOBALS['dbi']->fetchResult($query);
-            } else {
-                if ($GLOBALS['dbs_to_test'] === false) {
-                    $query = "SHOW DATABASES ";
-                    $query .= $this->_getWhereClause('Database', $searchClause);
-                    $handle = $GLOBALS['dbi']->tryQuery($query);
-                    if ($handle !== false) {
-                        $prefixMap = array();
-                        $total = $pos + $maxItems;
-                        while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                            $prefix = strstr($arr[0], $dbSeparator, true);
-                            if ($prefix === false) {
-                                $prefix = $arr[0];
-                            }
-                            $prefixMap[$prefix] = 1;
-                            if (sizeof($prefixMap) == $total) {
-                                break;
-                            }
-                        }
-                        $prefixes = array_slice(array_keys($prefixMap), $pos);
-                    }
-
-                    $query = "SHOW DATABASES ";
-                    $query .= $this->_getWhereClause('Database', '');
-                    $query .= " AND (";
-                    $subClauses = array();
-                    foreach ($prefixes as $prefix) {
-                        $subClauses[] = " LOCATE('"
-                            . PMA_Util::sqlAddSlashes($prefix) . $dbSeparator . "', "
-                            . "CONCAT(`Database`, '" . $dbSeparator . "')) = 1 ";
-                    }
-                    $query .= implode("OR", $subClauses) . ")";
-                    $retval = $GLOBALS['dbi']->fetchResult($query);
-                } else {
-                    $retval = array();
-                    $prefixMap = array();
-                    $total = $pos + $maxItems;
-                    foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
-                        $query = "SHOW DATABASES LIKE '" . $db . "'";
-                        $handle = $GLOBALS['dbi']->tryQuery($query);
-                        if ($handle !== false) {
-                            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                                if ($this->_isHideDb($arr[0])) {
-                                    continue;
-                                }
-                                $prefix = strstr($arr[0], $dbSeparator, true);
-                                if ($prefix === false) {
-                                    $prefix = $arr[0];
-                                }
-                                $prefixMap[$prefix] = 1;
-                                if (sizeof($prefixMap) == $total) {
-                                    break 2;
-                                }
-                            }
-                        }
-                    }
-                    $prefixes = array_slice(array_keys($prefixMap), $pos);
-
-                    foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
-                        $query = "SHOW DATABASES LIKE '" . $db . "'";
-                        $handle = $GLOBALS['dbi']->tryQuery($query);
-                        if ($handle !== false) {
-                            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                                if ($this->_isHideDb($arr[0])) {
-                                    continue;
-                                }
-                                if (! in_array($arr[0], $retval)) {
-                                    foreach ($prefixes as $prefix) {
-                                        $starts_with = strpos(
-                                            $arr[0] . $dbSeparator,
-                                            $prefix . $dbSeparator
-                                        ) === 0;
-                                        if ($starts_with) {
-                                            $retval[] = $arr[0];
-                                            break;
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-                    sort($retval);
-                }
-            }
-        } else {
-            if (! $GLOBALS['cfg']['Server']['DisableIS']) {
+        if (!$GLOBALS['cfg']['NavigationTreeEnableGrouping']
+            || !$GLOBALS['cfg']['ShowDatabasesNavigationAsTree']
+        ) {
+            if (isset($GLOBALS['cfg']['Server']['DisableIS'])
+                && ! $GLOBALS['cfg']['Server']['DisableIS']
+            ) {
                 $query  = "SELECT `SCHEMA_NAME` ";
                 $query .= "FROM `INFORMATION_SCHEMA`.`SCHEMATA` ";
                 $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
                 $query .= "ORDER BY `SCHEMA_NAME` ";
                 $query .= "LIMIT $pos, $maxItems";
                 $retval = $GLOBALS['dbi']->fetchResult($query);
-            } else {
-                if ($GLOBALS['dbs_to_test'] === false) {
-                    $retval = array();
-                    $query = "SHOW DATABASES ";
-                    $query .= $this->_getWhereClause('Database', $searchClause);
-                    $handle = $GLOBALS['dbi']->tryQuery($query);
-                    if ($handle !== false) {
-                        $count = 0;
-                        while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                            if ($pos <= 0 && $count < $maxItems) {
-                                $retval[] = $arr[0];
-                                $count++;
-                            }
-                            $pos--;
-                        }
-                    }
-                } else {
-                    $retval = array();
+                return $retval;
+            }
+
+            if ($GLOBALS['dbs_to_test'] === false) {
+                $retval = array();
+                $query = "SHOW DATABASES ";
+                $query .= $this->_getWhereClause('Database', $searchClause);
+                $handle = $GLOBALS['dbi']->tryQuery($query);
+                if ($handle !== false) {
                     $count = 0;
-                    foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
-                        $query = "SHOW DATABASES LIKE '" . $db . "'";
-                        $handle = $GLOBALS['dbi']->tryQuery($query);
-                        if ($handle !== false) {
-                            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                                if ($this->_isHideDb($arr[0])) {
-                                    continue;
-                                }
-                                if (! in_array($arr[0], $retval)) {
-                                    if ($pos <= 0 && $count < $maxItems) {
-                                        $retval[] = $arr[0];
-                                        $count++;
-                                    }
-                                    $pos--;
-                                }
-                            }
+                    while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                        if ($pos <= 0 && $count < $maxItems) {
+                            $retval[] = $arr[0];
+                            $count++;
                         }
+                        $pos--;
                     }
-                    sort($retval);
+                }
+                return $retval;
+            }
+
+            $retval = array();
+            $count = 0;
+            foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
+                $query = "SHOW DATABASES LIKE '" . $db . "'";
+                $handle = $GLOBALS['dbi']->tryQuery($query);
+                if ($handle === false) {
+                    continue;
+                }
+
+                while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                    if ($this->_isHideDb($arr[0])) {
+                        continue;
+                    }
+                    if (in_array($arr[0], $retval)) {
+                        continue;
+                    }
+
+                    if ($pos <= 0 && $count < $maxItems) {
+                        $retval[] = $arr[0];
+                        $count++;
+                    }
+                    $pos--;
+                }
+            }
+            sort($retval);
+            return $retval;
+        }
+
+        $dbSeparator = $GLOBALS['cfg']['NavigationTreeDbSeparator'];
+        if (isset($GLOBALS['cfg']['Server']['DisableIS'])
+            && ! $GLOBALS['cfg']['Server']['DisableIS']
+        ) {
+            $query  = "SELECT `SCHEMA_NAME` ";
+            $query .= "FROM `INFORMATION_SCHEMA`.`SCHEMATA`, ";
+            $query .= "(";
+            $query .= "SELECT DB_first_level ";
+            $query .= "FROM ( ";
+            $query .= "SELECT DISTINCT SUBSTRING_INDEX(SCHEMA_NAME, ";
+            $query .= "'$dbSeparator', 1) ";
+            $query .= "DB_first_level ";
+            $query .= "FROM INFORMATION_SCHEMA.SCHEMATA ";
+            $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
+            $query .= ") t ";
+            $query .= "ORDER BY DB_first_level ASC ";
+            $query .= "LIMIT $pos, $maxItems";
+            $query .= ") t2 ";
+            $query .= "WHERE 1 = LOCATE(CONCAT(DB_first_level, ";
+            $query .= "'$dbSeparator'), ";
+            $query .= "CONCAT(SCHEMA_NAME, ";
+            $query .= "'$dbSeparator')) ";
+            $query .= "ORDER BY SCHEMA_NAME ASC";
+            $retval = $GLOBALS['dbi']->fetchResult($query);
+            return $retval;
+        }
+
+        if ($GLOBALS['dbs_to_test'] === false) {
+            $query = "SHOW DATABASES ";
+            $query .= $this->_getWhereClause('Database', $searchClause);
+            $handle = $GLOBALS['dbi']->tryQuery($query);
+            $prefixes = array();
+            if ($handle !== false) {
+                $prefixMap = array();
+                $total = $pos + $maxItems;
+                while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                    $prefix = strstr($arr[0], $dbSeparator, true);
+                    if ($prefix === false) {
+                        $prefix = $arr[0];
+                    }
+                    $prefixMap[$prefix] = 1;
+                    if (sizeof($prefixMap) == $total) {
+                        break;
+                    }
+                }
+                $prefixes = array_slice(array_keys($prefixMap), $pos);
+            }
+
+            $query = "SHOW DATABASES ";
+            $query .= $this->_getWhereClause('Database', '');
+            $query .= " AND (";
+            $subClauses = array();
+            foreach ($prefixes as $prefix) {
+                $subClauses[] = " LOCATE('"
+                    . PMA_Util::sqlAddSlashes($prefix) . $dbSeparator . "', "
+                    . "CONCAT(`Database`, '" . $dbSeparator . "')) = 1 ";
+            }
+            $query .= implode("OR", $subClauses) . ")";
+            $retval = $GLOBALS['dbi']->fetchResult($query);
+            return $retval;
+        }
+
+        $retval = array();
+        $prefixMap = array();
+        $total = $pos + $maxItems;
+        foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
+            $query = "SHOW DATABASES LIKE '" . $db . "'";
+            $handle = $GLOBALS['dbi']->tryQuery($query);
+            if ($handle === false) {
+                continue;
+            }
+
+            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                if ($this->_isHideDb($arr[0])) {
+                    continue;
+                }
+                $prefix = strstr($arr[0], $dbSeparator, true);
+                if ($prefix === false) {
+                    $prefix = $arr[0];
+                }
+                $prefixMap[$prefix] = 1;
+                if (sizeof($prefixMap) == $total) {
+                    break 2;
                 }
             }
         }
+        $prefixes = array_slice(array_keys($prefixMap), $pos);
+
+        foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
+            $query = "SHOW DATABASES LIKE '" . $db . "'";
+            $handle = $GLOBALS['dbi']->tryQuery($query);
+            if ($handle === false) {
+                continue;
+            }
+
+            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                if ($this->_isHideDb($arr[0])) {
+                    continue;
+                }
+                if (in_array($arr[0], $retval)) {
+                    continue;
+                }
+
+                foreach ($prefixes as $prefix) {
+                    $starts_with = strpos(
+                        $arr[0] . $dbSeparator,
+                        $prefix . $dbSeparator
+                    ) === 0;
+                    if ($starts_with) {
+                        $retval[] = $arr[0];
+                        break;
+                    }
+                }
+            }
+        }
+        sort($retval);
 
         return $retval;
     }
@@ -536,79 +559,91 @@ class Node
      */
     public function getPresence($type = '', $searchClause = '')
     {
-        if ($GLOBALS['cfg']['NavigationTreeEnableGrouping']) {
-            $dbSeparator = $GLOBALS['cfg']['NavigationTreeDbSeparator'];
-            if (! $GLOBALS['cfg']['Server']['DisableIS']) {
+        if (!$GLOBALS['cfg']['NavigationTreeEnableGrouping']
+            || !$GLOBALS['cfg']['ShowDatabasesNavigationAsTree']
+        ) {
+            if (isset($GLOBALS['cfg']['Server']['DisableIS'])
+                && ! $GLOBALS['cfg']['Server']['DisableIS']
+            ) {
                 $query = "SELECT COUNT(*) ";
-                $query .= "FROM ( ";
-                $query .= "SELECT DISTINCT SUBSTRING_INDEX(SCHEMA_NAME, ";
-                $query .= "'$dbSeparator', 1) ";
-                $query .= "DB_first_level ";
                 $query .= "FROM INFORMATION_SCHEMA.SCHEMATA ";
                 $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
-                $query .= ") t ";
                 $retval = (int)$GLOBALS['dbi']->fetchValue($query);
-            } else {
-                if ($GLOBALS['dbs_to_test'] === false) {
-                    $prefixMap = array();
-                    $query = "SHOW DATABASES ";
-                    $query .= $this->_getWhereClause('Database', $searchClause);
-                    $handle = $GLOBALS['dbi']->tryQuery($query);
-                    if ($handle !== false) {
-                        while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                            $prefix = strstr($arr[0], $dbSeparator, true);
-                            if ($prefix === false) {
-                                $prefix = $arr[0];
-                            }
-                            $prefixMap[$prefix] = 1;
-                        }
+                return $retval;
+            }
+
+            if ($GLOBALS['dbs_to_test'] === false) {
+                $query = "SHOW DATABASES ";
+                $query .= $this->_getWhereClause('Database', $searchClause);
+                $retval = $GLOBALS['dbi']->numRows(
+                    $GLOBALS['dbi']->tryQuery($query)
+                );
+                return $retval;
+            }
+
+            $retval = 0;
+            foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
+                $query = "SHOW DATABASES LIKE '" . $db . "'";
+                $retval += $GLOBALS['dbi']->numRows(
+                    $GLOBALS['dbi']->tryQuery($query)
+                );
+            }
+            return $retval;
+        }
+
+        $dbSeparator = $GLOBALS['cfg']['NavigationTreeDbSeparator'];
+        if (! $GLOBALS['cfg']['Server']['DisableIS']) {
+            $query = "SELECT COUNT(*) ";
+            $query .= "FROM ( ";
+            $query .= "SELECT DISTINCT SUBSTRING_INDEX(SCHEMA_NAME, ";
+            $query .= "'$dbSeparator', 1) ";
+            $query .= "DB_first_level ";
+            $query .= "FROM INFORMATION_SCHEMA.SCHEMATA ";
+            $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
+            $query .= ") t ";
+            $retval = (int)$GLOBALS['dbi']->fetchValue($query);
+            return $retval;
+        }
+
+        if ($GLOBALS['dbs_to_test'] !== false) {
+            $prefixMap = array();
+            foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
+                $query = "SHOW DATABASES LIKE '" . $db . "'";
+                $handle = $GLOBALS['dbi']->tryQuery($query);
+                if ($handle === false) {
+                    continue;
+                }
+
+                while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                    if ($this->_isHideDb($arr[0])) {
+                        continue;
                     }
-                    $retval = count($prefixMap);
-                } else {
-                    $prefixMap = array();
-                    foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
-                        $query = "SHOW DATABASES LIKE '" . $db . "'";
-                        $handle = $GLOBALS['dbi']->tryQuery($query);
-                        if ($handle !== false) {
-                            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
-                                if ($this->_isHideDb($arr[0])) {
-                                    continue;
-                                }
-                                $prefix = strstr($arr[0], $dbSeparator, true);
-                                if ($prefix === false) {
-                                    $prefix = $arr[0];
-                                }
-                                $prefixMap[$prefix] = 1;
-                            }
-                        }
+                    $prefix = strstr($arr[0], $dbSeparator, true);
+                    if ($prefix === false) {
+                        $prefix = $arr[0];
                     }
-                    $retval = count($prefixMap);
+                    $prefixMap[$prefix] = 1;
                 }
             }
-        } else {
-            if (! $GLOBALS['cfg']['Server']['DisableIS']) {
-                $query = "SELECT COUNT(*) ";
-                $query .= "FROM INFORMATION_SCHEMA.SCHEMATA ";
-                $query .= $this->_getWhereClause('SCHEMA_NAME', $searchClause);
-                $retval = (int)$GLOBALS['dbi']->fetchValue($query);
-            } else {
-                if ($GLOBALS['dbs_to_test'] === false) {
-                    $query = "SHOW DATABASES ";
-                    $query .= $this->_getWhereClause('Database', $searchClause);
-                    $retval = $GLOBALS['dbi']->numRows(
-                        $GLOBALS['dbi']->tryQuery($query)
-                    );
-                } else {
-                    $retval = 0;
-                    foreach ($this->_getDatabasesToSearch($searchClause) as $db) {
-                        $query = "SHOW DATABASES LIKE '" . $db . "'";
-                        $retval += $GLOBALS['dbi']->numRows(
-                            $GLOBALS['dbi']->tryQuery($query)
-                        );
-                    }
+            $retval = count($prefixMap);
+            return $retval;
+        }
+
+        $prefixMap = array();
+        $query = "SHOW DATABASES ";
+        $query .= $this->_getWhereClause('Database', $searchClause);
+        $handle = $GLOBALS['dbi']->tryQuery($query);
+        if ($handle !== false) {
+            while ($arr = $GLOBALS['dbi']->fetchArray($handle)) {
+                $prefix = strstr($arr[0], $dbSeparator, true);
+                if ($prefix === false) {
+                    $prefix = $arr[0];
                 }
+                $prefixMap[$prefix] = 1;
             }
         }
+        $retval = count($prefixMap);
+
         return $retval;
     }
 
@@ -619,7 +654,8 @@ class Node
      *
      * @return boolean whether to hide
      */
-    private function _isHideDb($db) {
+    private function _isHideDb($db)
+    {
         if (! empty($GLOBALS['cfg']['Server']['hide_db'])
             && preg_match('/' . $GLOBALS['cfg']['Server']['hide_db'] . '/', $db)
         ) {
@@ -714,7 +750,8 @@ class Node
      */
     public function getCssClasses($match)
     {
-        if ($GLOBALS['cfg']['NavigationTreeDisableDatabaseExpansion']) {
+        if (! $GLOBALS['cfg']['NavigationTreeEnableExpansion']
+        ) {
             return '';
         }
 
@@ -739,9 +776,10 @@ class Node
      */
     public function getIcon($match)
     {
-        if ($GLOBALS['cfg']['NavigationTreeDisableDatabaseExpansion']) {
+        if (! $GLOBALS['cfg']['NavigationTreeEnableExpansion']
+        ) {
             return '';
-        } elseif ($match) {
+        } elseif ($match && ! $this->is_group) {
             $this->visible = true;
             return PMA_Util::getImage('b_minus.png');
         } else {
